@@ -11,7 +11,6 @@ from torch.utils.data import DataLoader, Dataset, TensorDataset
 from sklearn.metrics import (
     mean_absolute_error,
     mean_squared_error,
-    root_mean_squared_error,
     r2_score,
     classification_report
 )
@@ -89,6 +88,7 @@ class ModelWrapper:
         persistent_workers: bool = False,
         early_stopping_rounds: int = None,
         print_per_rounds: int = 1,
+        drop_last: bool = False,
         checkpoint_per_rounds: int = 0,
         checkpoint_name="model.pt",
         show_progress=True,
@@ -110,6 +110,7 @@ class ModelWrapper:
                 persistent_workers,
                 early_stopping_rounds,  # 早停，等10轮决策，评价指标不在变化，停止
                 print_per_rounds,
+                drop_last,
                 checkpoint_per_rounds,
                 checkpoint_name,
                 self.device,
@@ -131,6 +132,7 @@ class ModelWrapper:
                 persistent_workers,
                 early_stopping_rounds,  # 早停，等10轮决策，评价指标不在变化，停止
                 print_per_rounds,
+                drop_last,
                 checkpoint_per_rounds,
                 checkpoint_name,
                 self.device,
@@ -144,7 +146,7 @@ class ModelWrapper:
         size = len(X)
         X = self._convert_X_to_tensor(X).to(self.device)
         self.best_model.eval()
-        with torch.no_grad():
+        with torch.inference_mode():
             if size >= (batch_size << 1): 
                 chunks = size // batch_size if size % batch_size == 0 else size // batch_size + 1
                 preds = [self.best_model(x) for x in torch.chunk(X, chunks, dim=0)]
@@ -292,6 +294,7 @@ class SimpleModelWrapper(ModelWrapper):
         persistent_workers: bool = False,
         early_stopping_rounds: int = None,
         print_per_rounds: int = 1,
+        drop_last: bool = False,
         checkpoint_per_rounds: int = 0,
         checkpoint_name="model.pt",
         show_progress=True,
@@ -313,6 +316,7 @@ class SimpleModelWrapper(ModelWrapper):
                 persistent_workers,
                 early_stopping_rounds,
                 print_per_rounds,
+                drop_last,
                 checkpoint_per_rounds,
                 checkpoint_name,
                 self.device,
@@ -334,6 +338,7 @@ class SimpleModelWrapper(ModelWrapper):
                 persistent_workers,
                 early_stopping_rounds,  # 早停，等10轮决策，评价指标不在变化，停止
                 print_per_rounds,
+                drop_last,
                 checkpoint_per_rounds,
                 checkpoint_name,
                 self.device,
@@ -394,10 +399,12 @@ class ClassifyModelWrapper(ModelWrapper):
         persistent_workers: bool = False,
         early_stopping_rounds: int = None,
         print_per_rounds: int = 1,
+        drop_last: bool = False,
         checkpoint_per_rounds: int = 0,
         checkpoint_name="model.pt",
         show_progress=True,
         eps=1e-5,
+        monitor: str = "accuracy"
     ) -> dict:
         if val_set:
             trainer = EvalClassTrainer(
@@ -415,12 +422,13 @@ class ClassifyModelWrapper(ModelWrapper):
                 persistent_workers,
                 early_stopping_rounds,  # 早停，等10轮决策，评价指标不在变化，停止
                 print_per_rounds,
+                drop_last,
                 checkpoint_per_rounds,
                 checkpoint_name,
                 self.device,
             )
             self.best_model, histories = trainer.train(
-                self.model, train_set, val_set, collate_fn, show_progress, eps
+                self.model, train_set, val_set, collate_fn, show_progress, eps, monitor
             )
         else:
             trainer = ClassTrainer(
@@ -436,12 +444,13 @@ class ClassifyModelWrapper(ModelWrapper):
                 persistent_workers,
                 early_stopping_rounds,  # 早停，等10轮决策，评价指标不在变化，停止
                 print_per_rounds,
+                drop_last,
                 checkpoint_per_rounds,
                 checkpoint_name,
                 self.device,
             )
             self.best_model, histories = trainer.train(
-                self.model, train_set, collate_fn, show_progress, eps
+                self.model, train_set, collate_fn, show_progress, eps, monitor
             )
         return histories
 
@@ -606,10 +615,12 @@ class SimpleClassifyModelWrapper(ClassifyModelWrapper):
         persistent_workers: bool = False,
         early_stopping_rounds: int = None,
         print_per_rounds: int = 1,
+        drop_last: bool = False,
         checkpoint_per_rounds: int = 0,
         checkpoint_name="model.pt",
         show_progress=True,
         eps=1e-5,
+        monitor: str = "accuracy"
     ) -> dict:
         if val_data:
             trainer = SimpleEvalClassTrainer(
@@ -627,12 +638,13 @@ class SimpleClassifyModelWrapper(ClassifyModelWrapper):
                 persistent_workers,
                 early_stopping_rounds,
                 print_per_rounds,
+                drop_last,
                 checkpoint_per_rounds,
                 checkpoint_name,
                 self.device,
             )
             self.best_model, histories = trainer.train(
-                self.model, X, y, val_data, collate_fn, show_progress, eps
+                self.model, X, y, val_data, collate_fn, show_progress, eps, monitor
             )
         else:
             trainer = SimpleClassTrainer(
@@ -648,12 +660,13 @@ class SimpleClassifyModelWrapper(ClassifyModelWrapper):
                 persistent_workers,
                 early_stopping_rounds,
                 print_per_rounds,
+                drop_last,
                 checkpoint_per_rounds,
                 checkpoint_name,
                 self.device,
             )
             self.best_model, histories = trainer.train(
-                self.model, X, y, collate_fn, show_progress, eps
+                self.model, X, y, collate_fn, show_progress, eps, monitor
             )
         return histories
 
@@ -729,10 +742,12 @@ class SplitClassifyModelWrapper(SimpleClassifyModelWrapper):
         persistent_workers: bool = False,
         early_stopping_rounds: int = None,
         print_per_rounds: int = 1,
+        drop_last: bool = False,
         checkpoint_per_rounds: int = 0,
         checkpoint_name="model.pt",
         show_progress=True,
         eps=1e-5,
+        monitor: str = "accuracy"
     ) -> dict:
         X_train, X_test, y_train, y_test = train_test_split(
             X, y, test_size=val_size, random_state=random_state
@@ -752,6 +767,7 @@ class SplitClassifyModelWrapper(SimpleClassifyModelWrapper):
             persistent_workers,
             early_stopping_rounds,
             print_per_rounds,
+            drop_last,
             checkpoint_per_rounds,
             checkpoint_name,
             self.device,
@@ -764,6 +780,7 @@ class SplitClassifyModelWrapper(SimpleClassifyModelWrapper):
             collate_fn,
             show_progress,
             eps,
+            monitor
         )
         return histories
 
@@ -804,6 +821,7 @@ class RegressModelWrapper(ModelWrapper):
         persistent_workers: bool = False,
         early_stopping_rounds: int = None,
         print_per_rounds: int = 1,
+        drop_last: bool = False,
         checkpoint_per_rounds: int = 0,
         checkpoint_name="model.pt",
         show_progress=True,
@@ -826,6 +844,7 @@ class RegressModelWrapper(ModelWrapper):
                 persistent_workers,
                 early_stopping_rounds,  # 早停，等10轮决策，评价指标不在变化，停止
                 print_per_rounds,
+                drop_last,
                 checkpoint_per_rounds,
                 checkpoint_name,
                 self.device,
@@ -847,6 +866,7 @@ class RegressModelWrapper(ModelWrapper):
                 persistent_workers,
                 early_stopping_rounds,  # 早停，等10轮决策，评价指标不在变化，停止
                 print_per_rounds,
+                drop_last,
                 checkpoint_per_rounds,
                 checkpoint_name,
                 self.device,
@@ -862,7 +882,7 @@ class RegressModelWrapper(ModelWrapper):
     def mae(
         self,
         X: Union[torch.Tensor, np.ndarray, List[float]],
-        y: Union[torch.Tensor, np.ndarray, list[float]],
+        y: Union[torch.Tensor, np.ndarray, List[float]],
         batch_size=128
     ) -> float:
         if isinstance(y, list):
@@ -876,7 +896,7 @@ class RegressModelWrapper(ModelWrapper):
     def mse(
         self,
         X: Union[torch.Tensor, np.ndarray, List[float]],
-        y: Union[torch.Tensor, np.ndarray, list[float]],
+        y: Union[torch.Tensor, np.ndarray, List[float]],
         batch_size=128
     ) -> float:
         if isinstance(y, list):
@@ -890,16 +910,10 @@ class RegressModelWrapper(ModelWrapper):
     def rmse(
         self,
         X: Union[torch.Tensor, np.ndarray, List[float]],
-        y: Union[torch.Tensor, np.ndarray, list[float]],
+        y: Union[torch.Tensor, np.ndarray, List[float]],
         batch_size=128
     ) -> float:
-        if isinstance(y, list):
-            y = np.array(y)
-        elif torch.is_tensor(y):
-            y = y.numpy()
-
-        pred = self.predict(X, batch_size)
-        return root_mean_squared_error(y.ravel(), pred.ravel())
+        return np.sqrt(self.mse(X, y, batch_size))
 
     def evaluate(
         self, val_set: Dataset, batch_size=128, num_workers=0, collate_fn=None
@@ -917,7 +931,7 @@ class RegressModelWrapper(ModelWrapper):
     def metrics(
         self,
         X: Union[torch.Tensor, np.ndarray, List[float]],
-        y: Union[torch.Tensor, np.ndarray, list[float]],
+        y: Union[torch.Tensor, np.ndarray, List[float]],
         batch_size=128
     ) -> dict:
         if isinstance(y, list):
@@ -925,10 +939,11 @@ class RegressModelWrapper(ModelWrapper):
         elif torch.is_tensor(y):
             y = y.numpy()
         pred = self.predict(X, batch_size)
+        mse = mean_squared_error(y.ravel(), pred.ravel())
         return {
             "MAE": mean_absolute_error(y.ravel(), pred.ravel()),
-            "MSE": mean_squared_error(y.ravel(), pred.ravel()),
-            "RMSE": root_mean_squared_error(y.ravel(), pred.ravel()),
+            "MSE": mse,
+            "RMSE": np.sqrt(mse),
             "R2": r2_score(y.ravel(), pred.ravel()),
         }
 
@@ -972,6 +987,7 @@ class SimpleRegressModelWrapper(RegressModelWrapper):
         persistent_workers: bool = False,
         early_stopping_rounds: int = None,
         print_per_rounds: int = 1,
+        drop_last: bool = False,
         checkpoint_per_rounds: int = 0,
         checkpoint_name="model.pt",
         show_progress=True,
@@ -994,6 +1010,7 @@ class SimpleRegressModelWrapper(RegressModelWrapper):
                 persistent_workers,
                 early_stopping_rounds,
                 print_per_rounds,
+                drop_last,
                 checkpoint_per_rounds,
                 checkpoint_name,
                 self.device,
@@ -1015,6 +1032,7 @@ class SimpleRegressModelWrapper(RegressModelWrapper):
                 persistent_workers,
                 early_stopping_rounds,  # 早停，等10轮决策，评价指标不在变化，停止
                 print_per_rounds,
+                drop_last,
                 checkpoint_per_rounds,
                 checkpoint_name,
                 self.device,
@@ -1029,15 +1047,13 @@ class SimpleRegressModelWrapper(RegressModelWrapper):
         X: Union[torch.Tensor, np.ndarray, List, Tuple],
         y: Union[torch.LongTensor, np.ndarray, List],
         batch_size=128,
-        num_workers=0,
-        collate_fn=None,
     ) -> float:
         """return loss"""
         if isinstance(y, list):
             y = np.array(y)
         elif torch.is_tensor(y):
             y = y.numpy()
-        pred = self.predict(X)
+        pred = self.predict(X, batch_size)
         return r2_score(y.ravel(), pred.ravel())
 
 
@@ -1079,6 +1095,7 @@ class SplitRegressModelWrapper(SimpleRegressModelWrapper):
         persistent_workers: bool = False,
         early_stopping_rounds: int = None,
         print_per_rounds: int = 1,
+        drop_last: bool = False,
         checkpoint_per_rounds: int = 0,
         checkpoint_name="model.pt",
         show_progress=True,
@@ -1089,7 +1106,11 @@ class SplitRegressModelWrapper(SimpleRegressModelWrapper):
             X, y, test_size=val_size, random_state=random_state
         )
         val_data = (X_test, y_test)
-        trainer = SimpleEvalRegressTrainer(
+        return super().train(
+            X_train,
+            y_train,
+            val_data,
+            collate_fn,
             epochs,
             optimizer,
             scheduler,
@@ -1104,11 +1125,10 @@ class SplitRegressModelWrapper(SimpleRegressModelWrapper):
             persistent_workers,
             early_stopping_rounds,
             print_per_rounds,
+            drop_last,
             checkpoint_per_rounds,
             checkpoint_name,
-            self.device,
+            show_progress,
+            eps,
+            monitor
         )
-        self.best_model, histories = trainer.train(
-            self.model, X, y, val_data, collate_fn, show_progress, eps, monitor
-        )
-        return histories
