@@ -29,11 +29,18 @@ from ._support import (
     train_epoch_acc,
     train_epoch_r2,
 )
-from model_wrapper.dataset import ListDataset, ListTensorCollector
+from model_wrapper.dataset import ListDataset
+from model_wrapper.collator import ListTensorCollator
+TRAIN_LOSS = 'train_loss'
+VAL_LOSS = 'val_loss'
+TRAIN_ACC = 'train_acc'
+VAL_ACC = 'val_acc'
+TRAIN_R2 = 'train_r2'
+VAL_R2 = 'val_r2'
 
 
 class Trainer:
-
+    
     def __init__(
         self,
         epochs=100,
@@ -48,6 +55,7 @@ class Trainer:
         persistent_workers: bool = False,
         early_stopping_rounds: int = None,  # 早停，等10轮决策，评价指标不在变化，停止
         print_per_rounds: int = 1,
+        drop_last: bool = False,
         checkpoint_per_rounds: int = 0,
         checkpoint_name="model.pt",
         device: Union[str, int, torch.device] = "auto",
@@ -67,6 +75,7 @@ class Trainer:
             epochs
         )
         self.print_per_rounds = print_per_rounds
+        self.drop_last = drop_last
         self.checkpoint_per_rounds = checkpoint_per_rounds
         self.checkpoint_name = checkpoint_name
 
@@ -108,6 +117,7 @@ class Trainer:
             num_workers=num_workers,
             shuffle=True,
             collate_fn=collate_fn,
+            drop_last=self.drop_last
         )
 
     def _build_val_loader(
@@ -126,6 +136,11 @@ class Trainer:
             num_workers=num_eval_workers,
             collate_fn=collate_fn,
         )
+    
+    def _check_device(self, model):
+        if next(model.parameters()).device != self.device:
+            model = model.to(self.device)
+        return model
 
     def train(
         self, model, train_set: Dataset, collate_fn=None, show_progress=False, eps=1e-5
@@ -136,8 +151,7 @@ class Trainer:
         min_loss = float("inf")
         best_model = None
         train_losses = []
-        if next(model.parameters()).device != self.device:
-            model = model.to(self.device)
+        model = self._check_device(model)
         optimizer, scheduler = self.get_optimizer_scheduler(model)
         is_tuple_params = isinstance(next(iter(train_loader)), (tuple, list))
 
@@ -176,7 +190,7 @@ class Trainer:
                 break
 
         log_utils.info(f"Min Loss: {min_loss:.4f}\n")
-        return best_model, {"train_losses": train_losses}
+        return best_model, {TRAIN_LOSS: train_losses}
 
     def get_optimizer_scheduler(self, model):
         scheduler = None
@@ -229,6 +243,7 @@ class SimpleTrainer(Trainer):
         persistent_workers: bool = False,
         early_stopping_rounds: int = None,  # 早停，等10轮决策，评价指标不在变化，停止
         print_per_rounds: int = 1,
+        drop_last: bool = False,
         checkpoint_per_rounds: int = 0,
         checkpoint_name="model.pt",
         device: Union[str, int, torch.device] = "auto",
@@ -246,6 +261,7 @@ class SimpleTrainer(Trainer):
             persistent_workers,
             early_stopping_rounds,
             print_per_rounds,
+            drop_last,
             checkpoint_per_rounds,
             checkpoint_name,
             device,
@@ -263,7 +279,7 @@ class SimpleTrainer(Trainer):
                 )
             else:
                 return super().train(
-                    model, ListDataset(*X, y), collate_fn if collate_fn else ListTensorCollector(), show_progress, eps=eps
+                    model, ListDataset(*X, y), collate_fn if collate_fn else ListTensorCollator(), show_progress, eps=eps
                 )
 
         else:
@@ -291,6 +307,7 @@ class EvalTrainer(Trainer):
         persistent_workers: bool = False,
         early_stopping_rounds: int = None,  # 早停，等10轮决策，评价指标不在变化，停止
         print_per_rounds: int = 1,
+        drop_last: bool = False,
         checkpoint_per_rounds: int = 0,
         checkpoint_name="model.pt",
         device: Union[str, int, torch.device] = "auto",
@@ -308,6 +325,7 @@ class EvalTrainer(Trainer):
             persistent_workers,
             early_stopping_rounds,
             print_per_rounds,
+            drop_last,
             checkpoint_per_rounds,
             checkpoint_name,
             device,
@@ -333,8 +351,7 @@ class EvalTrainer(Trainer):
         best_model = None
         min_loss = float("inf")
         train_losses, val_losses = [], []
-        if next(model.parameters()).device != self.device:
-            model = model.to(self.device)
+        model = self._check_device(model)
         optimizer, scheduler = self.get_optimizer_scheduler(model)
         is_tuple_params = isinstance(next(iter(train_loader)), (tuple, list))
 
@@ -386,7 +403,7 @@ class EvalTrainer(Trainer):
 
 
         log_utils.info(f"Min Loss: {min_loss:.4f}\n")
-        return best_model, {"train_losses": train_losses, "val_losses": val_losses}
+        return best_model, {TRAIN_LOSS: train_losses, VAL_LOSS: val_losses}
 
     def print(self, show_progress, epoch, lr, loss, val_loss):
         if not show_progress:
@@ -413,6 +430,7 @@ class SimpleEvalTrainer(EvalTrainer):
         persistent_workers: bool = False,
         early_stopping_rounds: int = None,  # 早停，等10轮决策，评价指标不在变化，停止
         print_per_rounds: int = 1,
+        drop_last: bool = False,
         checkpoint_per_rounds: int = 0,
         checkpoint_name="model.pt",
         device: Union[str, int, torch.device] = "auto",
@@ -432,6 +450,7 @@ class SimpleEvalTrainer(EvalTrainer):
             persistent_workers,
             early_stopping_rounds,
             print_per_rounds,
+            drop_last,
             checkpoint_per_rounds,
             checkpoint_name,
             device,
@@ -466,7 +485,7 @@ class SimpleEvalTrainer(EvalTrainer):
                     model,
                     ListDataset(*X, y),
                     ListDataset(*val_data[0], val_data[1]),
-                    collate_fn if collate_fn else ListTensorCollector(),
+                    collate_fn if collate_fn else ListTensorCollator(),
                     show_progress,
                     eps,
                 )
@@ -499,6 +518,7 @@ class ClassTrainer(Trainer):
         persistent_workers: bool = False,
         early_stopping_rounds: int = None,  # 早停，等10轮决策，评价指标不在变化，停止
         print_per_rounds: int = 1,
+        drop_last: bool = False,
         checkpoint_per_rounds: int = 0,
         checkpoint_name="model.pt",
         device: Union[str, int, torch.device] = "auto",
@@ -516,6 +536,7 @@ class ClassTrainer(Trainer):
             persistent_workers,
             early_stopping_rounds,
             print_per_rounds,
+            drop_last,
             checkpoint_per_rounds,
             checkpoint_name,
             device,
@@ -536,8 +557,7 @@ class ClassTrainer(Trainer):
         min_loss = float("inf")
         cnt, cnt2, best_acc, last_acc = 0, 0, 0.0, 0.0
         train_losses, train_accs = [], []
-        if next(model.parameters()).device != self.device:
-            model = model.to(self.device)
+        model = self._check_device(model)
         optimizer, scheduler = self.get_optimizer_scheduler(model)
         is_tuple_params = isinstance(next(iter(train_loader)), (tuple, list))
         monitor = monitor.lower() in {"accuracy", "acc"}
@@ -567,11 +587,12 @@ class ClassTrainer(Trainer):
             )
             self.try_checkpoint(model, epoch)
 
-            if monitor and is_improve(best_acc, acc, min_loss, avg_loss, eps):
-                cnt, cnt2 = 0, 0
-                best_acc, best_model = acc, deepcopy(model)
-                last_acc, min_loss = acc, min(min_loss, avg_loss)
-                continue
+            if monitor:
+                if is_improve(best_acc, acc, min_loss, avg_loss, eps):
+                    cnt, cnt2 = 0, 0
+                    best_acc, best_model = acc, deepcopy(model)
+                    last_acc, min_loss = acc, min(min_loss, avg_loss)
+                    continue
             elif is_improve_loss(best_acc, acc, min_loss, avg_loss, eps):
                 cnt, cnt2 = 0, 0
                 min_loss, best_model = avg_loss, deepcopy(model)
@@ -594,7 +615,7 @@ class ClassTrainer(Trainer):
             min_loss = min(min_loss, avg_loss)
 
         log_utils.info(f"Best Accuracy: {best_acc:.2%}, Min Loss: {min_loss:.4f}\n")
-        return best_model, {"train_losses": train_losses, "train_accs": train_accs}
+        return best_model, {TRAIN_LOSS: train_losses, TRAIN_ACC: train_accs}
 
     def print(self, show_progress, epoch, lr, loss, acc):
         if not show_progress:
@@ -619,6 +640,7 @@ class SimpleClassTrainer(ClassTrainer):
         persistent_workers: bool = False,
         early_stopping_rounds: int = None,  # 早停，等10轮决策，评价指标不在变化，停止
         print_per_rounds: int = 1,
+        drop_last: bool = False,
         checkpoint_per_rounds: int = 0,
         checkpoint_name="model.pt",
         device: Union[str, int, torch.device] = "auto",
@@ -636,6 +658,7 @@ class SimpleClassTrainer(ClassTrainer):
             persistent_workers,
             early_stopping_rounds,
             print_per_rounds,
+            drop_last,
             checkpoint_per_rounds,
             checkpoint_name,
             device,
@@ -660,7 +683,7 @@ class SimpleClassTrainer(ClassTrainer):
                 )
             else:
                 return super().train(
-                    model, ListDataset(*X, y), collate_fn if collate_fn else ListTensorCollector(), show_progress, eps=eps
+                    model, ListDataset(*X, y), collate_fn if collate_fn else ListTensorCollator(), show_progress, eps=eps
                 )
 
         else:
@@ -693,6 +716,7 @@ class EvalClassTrainer(EvalTrainer):
         persistent_workers: bool = False,
         early_stopping_rounds: int = None,  # 早停，等10轮决策，评价指标不在变化，停止
         print_per_rounds: int = 1,
+        drop_last: bool = False,
         checkpoint_per_rounds: int = 0,
         checkpoint_name="model.pt",
         device: Union[str, int, torch.device] = "auto",
@@ -712,6 +736,7 @@ class EvalClassTrainer(EvalTrainer):
             persistent_workers,
             early_stopping_rounds,
             print_per_rounds,
+            drop_last,
             checkpoint_per_rounds,
             checkpoint_name,
             device,
@@ -736,8 +761,7 @@ class EvalClassTrainer(EvalTrainer):
         min_loss = float("inf")
         cnt, cnt2, best_acc, last_acc = 0, 0, 0.0, 0.0
         train_losses, train_accs, val_losses, val_accs = [], [], [], []
-        if next(model.parameters()).device != self.device:
-            model = model.to(self.device)
+        model = self._check_device(model)
         optimizer, scheduler = self.get_optimizer_scheduler(model)
         is_tuple_params = isinstance(next(iter(train_loader)), (tuple, list))
         monitor = monitor.lower() in {"accuracy", "acc"}
@@ -779,13 +803,12 @@ class EvalClassTrainer(EvalTrainer):
             )
             self.try_checkpoint(model, epoch)
 
-            if monitor and is_improve(
-                best_acc, val_acc, min_loss, val_loss, eps
-            ):
-                cnt, cnt2 = 0, 0
-                best_acc, best_model = val_acc, deepcopy(model)
-                last_acc, min_loss = val_acc, min(min_loss, val_loss)
-                continue
+            if monitor:
+                if is_improve(best_acc, val_acc, min_loss, val_loss, eps):
+                    cnt, cnt2 = 0, 0
+                    best_acc, best_model = val_acc, deepcopy(model)
+                    last_acc, min_loss = val_acc, min(min_loss, val_loss)
+                    continue
             elif is_improve_loss(best_acc, val_acc, min_loss, val_loss, eps):
                 cnt, cnt2 = 0, 0
                 min_loss, best_model = val_loss, deepcopy(model)
@@ -809,10 +832,10 @@ class EvalClassTrainer(EvalTrainer):
 
         log_utils.info(f"Best Accuracy: {best_acc:.2%}, Min Loss: {min_loss:.4f}\n")
         return best_model, {
-            "train_losses": train_losses,
-            "train_accs": train_accs,
-            "val_losses": val_losses,
-            "val_accs": val_accs,
+            TRAIN_LOSS: train_losses,
+            TRAIN_ACC: train_accs,
+            VAL_LOSS: val_losses,
+            VAL_ACC: val_accs,
         }
 
     def print(self, show_progress, epoch, lr, loss, acc, val_loss, val_acc):
@@ -841,6 +864,7 @@ class SimpleEvalClassTrainer(EvalClassTrainer):
         persistent_workers: bool = False,
         early_stopping_rounds: int = None,  # 早停，等10轮决策，评价指标不在变化，停止
         print_per_rounds: int = 1,
+        drop_last: bool = False,
         checkpoint_per_rounds: int = 0,
         checkpoint_name="model.pt",
         device: Union[str, int, torch.device] = "auto",
@@ -860,6 +884,7 @@ class SimpleEvalClassTrainer(EvalClassTrainer):
             persistent_workers,
             early_stopping_rounds,
             print_per_rounds,
+            drop_last,
             checkpoint_per_rounds,
             checkpoint_name,
             device,
@@ -896,7 +921,7 @@ class SimpleEvalClassTrainer(EvalClassTrainer):
                     model,
                     ListDataset(*X, y),
                     ListDataset(*val_data[0], val_data[1]),
-                    collate_fn if collate_fn else ListTensorCollector(),
+                    collate_fn if collate_fn else ListTensorCollator(),
                     show_progress,
                     eps,
                     monitor=monitor
@@ -931,6 +956,7 @@ class RegressTrainer(Trainer):
         persistent_workers: bool = False,
         early_stopping_rounds: int = None,  # 早停，等10轮决策，评价指标不在变化，停止
         print_per_rounds: int = 1,
+        drop_last: bool = False,
         checkpoint_per_rounds: int = 0,
         checkpoint_name="model.pt",
         device: Union[str, int, torch.device] = "auto",
@@ -948,10 +974,11 @@ class RegressTrainer(Trainer):
             persistent_workers,
             early_stopping_rounds,
             print_per_rounds,
+            drop_last,
             checkpoint_per_rounds,
             checkpoint_name,
             device,
-        )
+        )   
 
     def train(
         self, model, train_set: Dataset, collate_fn=None, show_progress=False, eps=1e-5
@@ -962,8 +989,7 @@ class RegressTrainer(Trainer):
         min_loss = float("inf")
         cnt, cnt2, best_r2, last_r2 = 0, 0, -1.0, -1.0
         train_losses, train_r2s = [], []
-        if next(model.parameters()).device != self.device:
-            model = model.to(self.device)
+        model = self._check_device(model)
         optimizer, scheduler = self.get_optimizer_scheduler(model)
         is_tuple_params = isinstance(next(iter(train_loader)), (tuple, list))
         monitor = monitor.lower() in {"r2_score", "r2"}
@@ -993,11 +1019,12 @@ class RegressTrainer(Trainer):
             )
             self.try_checkpoint(model, epoch)
 
-            if monitor and is_improve(best_r2, r2, min_loss, avg_loss, eps):
-                cnt, cnt2 = 0, 0
-                best_r2, best_model = r2, deepcopy(model)
-                last_r2, min_loss = r2, min(min_loss, avg_loss)
-                continue
+            if monitor: 
+                if is_improve(best_r2, r2, min_loss, avg_loss, eps):
+                    cnt, cnt2 = 0, 0
+                    best_r2, best_model = r2, deepcopy(model)
+                    last_r2, min_loss = r2, min(min_loss, avg_loss)
+                    continue
             elif is_improve_loss(best_r2, r2, min_loss, avg_loss, eps):
                 cnt, cnt2 = 0, 0
                 min_loss, best_model = avg_loss, deepcopy(model)
@@ -1020,7 +1047,7 @@ class RegressTrainer(Trainer):
             min_loss = min(min_loss, avg_loss)
 
         log_utils.info(f"Best R2 Score: {best_r2:.4f}, Min Loss: {min_loss:.4f}\n")
-        return best_model, {"train_losses": train_losses, "train_r2s": train_r2s}
+        return best_model, {TRAIN_LOSS: train_losses, TRAIN_R2: train_r2s}
 
     def print(self, show_progress, epoch, lr, loss, r2):
         if not show_progress:
@@ -1045,6 +1072,7 @@ class SimpleRegressTrainer(RegressTrainer):
         persistent_workers: bool = False,
         early_stopping_rounds: int = None,  # 早停，等10轮决策，评价指标不在变化，停止
         print_per_rounds: int = 1,
+        drop_last: bool = False,
         checkpoint_per_rounds: int = 0,
         checkpoint_name="model.pt",
         device: Union[str, int, torch.device] = "auto",
@@ -1062,6 +1090,7 @@ class SimpleRegressTrainer(RegressTrainer):
             persistent_workers,
             early_stopping_rounds,
             print_per_rounds,
+            drop_last,
             checkpoint_per_rounds,
             checkpoint_name,
             device,
@@ -1086,7 +1115,7 @@ class SimpleRegressTrainer(RegressTrainer):
                 )
             else:
                 return super().train(
-                    model, ListDataset(*X, y), collate_fn if collate_fn else ListTensorCollector(), show_progress, eps=eps
+                    model, ListDataset(*X, y), collate_fn if collate_fn else ListTensorCollator(), show_progress, eps=eps
                 )
 
         else:
@@ -1119,6 +1148,7 @@ class EvalRegressTrainer(Trainer):
         persistent_workers: bool = False,
         early_stopping_rounds: int = None,  # 早停，等10轮决策，评价指标不在变化，停止
         print_per_rounds: int = 1,
+        drop_last: bool = False,
         checkpoint_per_rounds: int = 0,
         checkpoint_name="model.pt",
         device: Union[str, int, torch.device] = "auto",
@@ -1136,6 +1166,7 @@ class EvalRegressTrainer(Trainer):
             persistent_workers,
             early_stopping_rounds,
             print_per_rounds,
+            drop_last,
             checkpoint_per_rounds,
             checkpoint_name,
             device,
@@ -1162,8 +1193,7 @@ class EvalRegressTrainer(Trainer):
         min_loss = float("inf")
         cnt, cnt2, best_r2, last_r2 = 0, 0, -1.0, -1.0
         train_losses, train_r2s, val_losses, val_r2s = [], [], [], []
-        if next(model.parameters()).device != self.device:
-            model = model.to(self.device)
+        model = self._check_device(model)
         optimizer, scheduler = self.get_optimizer_scheduler(model)
         is_tuple_params = isinstance(next(iter(train_loader)), (tuple, list))
         monitor = monitor.lower() in {"r2_score", "r2"}
@@ -1207,11 +1237,12 @@ class EvalRegressTrainer(Trainer):
             )
             self.try_checkpoint(model, epoch)
 
-            if monitor and is_improve(best_r2, val_r2, min_loss, val_loss, eps):
-                cnt, cnt2 = 0, 0
-                best_r2, best_model = val_r2, deepcopy(model)
-                last_r2, min_loss = val_r2, min(min_loss, val_loss)
-                continue
+            if monitor:
+                if is_improve(best_r2, val_r2, min_loss, val_loss, eps):
+                    cnt, cnt2 = 0, 0
+                    best_r2, best_model = val_r2, deepcopy(model)
+                    last_r2, min_loss = val_r2, min(min_loss, val_loss)
+                    continue
             elif is_improve_loss(best_r2, val_r2, min_loss, val_loss, eps):
                 cnt, cnt2 = 0, 0
                 min_loss, best_model = val_loss, deepcopy(model)
@@ -1235,10 +1266,10 @@ class EvalRegressTrainer(Trainer):
 
         log_utils.info(f"Best R2 Score: {best_r2:.4f}, Min Loss: {min_loss:.4f}\n")
         return best_model, {
-            "train_losses": train_losses,
-            "train_r2s": train_r2s,
-            "val_losses": val_losses,
-            "val_r2s": val_r2s,
+            TRAIN_LOSS: train_losses,
+            TRAIN_R2: train_r2s,
+            VAL_LOSS: val_losses,
+            VAL_R2: val_r2s,
         }
 
     def print(self, show_progress, epoch, lr, loss, r2, val_loss, val_r2):
@@ -1267,6 +1298,7 @@ class SimpleEvalRegressTrainer(EvalRegressTrainer):
         persistent_workers: bool = False,
         early_stopping_rounds: int = None,  # 早停，等10轮决策，评价指标不在变化，停止
         print_per_rounds: int = 1,
+        drop_last: bool = False,
         checkpoint_per_rounds: int = 0,
         checkpoint_name="model.pt",
         device: Union[str, int, torch.device] = "auto",
@@ -1286,6 +1318,7 @@ class SimpleEvalRegressTrainer(EvalRegressTrainer):
             persistent_workers,
             early_stopping_rounds,
             print_per_rounds,
+            drop_last,
             checkpoint_per_rounds,
             checkpoint_name,
             device,
@@ -1321,7 +1354,7 @@ class SimpleEvalRegressTrainer(EvalRegressTrainer):
                     model,
                     ListDataset(*X, y),
                     ListDataset(*val_data[0], val_data[1]),
-                    collate_fn if collate_fn else ListTensorCollector(),
+                    collate_fn if collate_fn else ListTensorCollator(),
                     show_progress,
                     eps,
                 )
